/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AbstractBindingComponent.java 601 2006-06-13 12:53:50Z wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.basic;

import java.net.URI;
import java.util.logging.Logger;

import javax.jbi.management.DeploymentException;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.petals.component.common.MEPConstants;

/**
 * This class provides a basic binding component, allowing developper to create
 * simply a new JBI binding component
 * 
 * @author wjoseph - eBMWebSourcing
 * 
 */
public abstract class AbstractBindingComponent extends AbstractBasicComponent {

    /**
     * Default contructor
     */
    public AbstractBindingComponent() {
        super();
    }

    /**
     * Create a new binding component, setting its logger and channel
     * 
     * @param channel
     *            the component's DeliveryChannel
     * @param log
     *            the component's logger
     */
    public AbstractBindingComponent(DeliveryChannel channel, Logger log) {
        super(channel, log);
    }

    /**
     * Process incoming messages for the component Ignore DONE,ERROR status.
     * Ignore "consumer" role.
     */
    public boolean process(MessageExchange exchange) throws Exception {
        if (ExchangeStatus.ACTIVE.equals(exchange.getStatus())
                && MessageExchange.Role.PROVIDER.equals(exchange.getRole())) {
            String address = ((SimpleBindingComponentServiceUnitManager) serviceUnitManager)
                    .getAddressForEndpoint(exchange.getEndpoint());
            if (MEPConstants.IN_ONLY_PATTERN.equals(exchange.getPattern())
                    || MEPConstants.ROBUST_IN_ONLY_PATTERN.equals(exchange
                            .getPattern())) {
                NormalizedMessage out = createNormalizedMessage(exchange);
                NormalizedMessage in = exchange.getMessage("in");
                sendMessage(address, in, out, exchange.getPattern());
                exchange.setStatus(ExchangeStatus.DONE);
            } else if (MEPConstants.IN_OUT_PATTERN
                    .equals(exchange.getPattern())
                    || MEPConstants.IN_OPTIONAL_OUT_PATTERN.equals(exchange
                            .getPattern())) {
                NormalizedMessage out = exchange.createMessage();
                NormalizedMessage in = exchange.getMessage("in");
                try {
                    if (sendMessage(address, in, out, exchange.getPattern())) {
                        exchange.setMessage(out, "out");
                    }
                } catch (Exception e) {
                    exchange.setFault(createFault(new Exception(
                            "Error sending the message", e), exchange));
                }
            } else {
                Exception e = new Exception(
                        "MessageExchangePattern not recognized :"
                                + exchange.getPattern());
                exchange.setError(e);
                channel.send(exchange);
                throw e;
            }
            channel.send(exchange);
        }
        return true;
    }

    /**
     * Send the incoming message to the outside component. The address
     * associated with the requested endpoint is resolved by the abstract
     * component. This is the only method a component needs to implement.
     * 
     * @param address
     *            the address associated with the endpoint
     * @param in
     *            the "in" message from the MessageExchange
     * @param out
     *            the "out" message from the MessageExchange
     * @param messageExchangePattern
     *            The exchange pattern of the MessageExchange
     * @return true if the out message has to be placed in the MessageExchange
     * @throws MessagingException
     *             If an error occurs during processing
     */
    public abstract boolean sendMessage(String address, NormalizedMessage in,
            NormalizedMessage out, URI messageExchangePattern)
        throws MessagingException;

    /**
     * Create a new SimpleBindingComponentServiceUnitManager
     * 
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.basic.AbstractBasicComponent#createServiceUnitManager()
     */
    @Override
    protected AbstractServiceUnitManager createServiceUnitManager()
        throws DeploymentException {
        return new SimpleBindingComponentServiceUnitManager(context, log);
    }
    
    /**
     * Send a message to the endpoint associated to the distant address
     * 
     * @param address
     *            the distant address of the requested service
     * @param messageExchange
     *            the message exchange to be sent
     * @throws MessagingException
     *             if the address has no endpoint associated to it, or if an
     *             error occurs while sending the message on the container
     */
    public void sendMessage(String address, MessageExchange messageExchange)
        throws MessagingException {
        ServiceEndpoint ep = ((SimpleBindingComponentServiceUnitManager) serviceUnitManager)
                .getEndpointForAddress(address);
        if (ep != null) {
            messageExchange.setEndpoint(ep);
            messageExchange.setService(ep.getServiceName());
            channel.send(messageExchange);
        } else {
            throw new MessagingException(
                    "No endpoint associated with this address");
        }
    }
}
