/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JmsConnexionFactory.java 601 2006-06-13 12:53:50Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.jms;

import java.io.File;
import java.io.FileFilter;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.spi.InitialContextFactory;
import static org.objectweb.petals.component.common.xmlmanipulation.StringHelper.*;

import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * This class is used to create <code>JmsConnexion</code> object.
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 */
public class JmsConnectionFactory {

    protected static JmsConnectionFactory instance = new JmsConnectionFactory();

    protected JmsConnectionFactory() {
    }

    protected JarFileFilter jarFileFilter = new JarFileFilter();

    protected WsdlFileFilter wsdlFileFilter = new WsdlFileFilter();

    protected static final String URI_SEPARATOR = ";";

    protected static final String URI_INITIAL_CONTEXT_FACTORY = "initialcontextfactory";

    protected static final String URI_TOPIC_CONNEXION_FACTORY = "tcf";

    protected static final String URI_QUEUE_CONNEXION_FACTORY = "qcf";

    protected static final String JBI_ADDRESS_TAG = "address";

    public static final String EXTENSIONS_XMLNS = "http://petals.objectweb.org/extensions";

    public static JmsConnectionFactory instance() {
        return instance;
    }

    /**
     * Create a Jms connection by extracting the serviceAddress
     * element in the wsdl file found in the specified directory.
     * The serviceAddress is parsed in order to retreive all information
     * about the connection (jndi properties, queue/topic name, ...)
     * @param serviceUnitRootPath
     * @return
     * @throws Exception WSDL file reading access problem, or serviceAddress not valid
     */
    public JmsConnection createConnection(String serviceUnitRootPath)
        throws Exception {
        JmsConnection conn = null;

        // 1. retreive the document related to the jbi.xml file
        Document jbiDef = findJBIDescription(serviceUnitRootPath);

        // 2. retreive in the WSDL file the 'address' of the service,
        //    i.e. the URI containing information about how to connect
        //    to the Queue / Topic
        URI address = extractAddressFromJbiDescriptor(jbiDef);

        // 3. extract JNDI properties from URI
        Properties props = extractJNDIProperties(address);

        // 4. create a class loader that will contains all JAR
        //    It will load the specific JndiFactory class
        //    to access the jndi containing Queue and Topic references
        URLClassLoader classLoader = createClassLoader(serviceUnitRootPath);

        // 5. load the InitialContext
        //InitialContextLoader loader = (InitialContextLoader)classLoader.loadClass(InitialContextLoader.class.getCanonicalName()).newInstance();
        //InitialContext ctx = loader.loadInitialContext(props);
        InitialContextFactory icf = (InitialContextFactory) classLoader
            .loadClass(
                props.getProperty(InitialContext.INITIAL_CONTEXT_FACTORY))
            .newInstance();
        Context ctx = icf.getInitialContext(props);

        // 6. get queue or topic name
        String name = extractName(address);

        // 7. get topicConn factory (can be null)
        String tCF = extractTopicConnexionFactory(address);

        // 8. get queueConn factory (can be null)
        String qCF = extractQueueConnexionFactory(address);

        // 9. Create the JmsConnexion
        conn = new JmsConnection(ctx, name, tCF, qCF);

        return conn;
    }

    /**
     * Search in the document the address (endpoint).
     * This address must be a valid URI defining all information
     * to find the queue or topic.
     * @param jbiDoc
     * @return the URI address
     * @throws Exception JBI document manipulation problem or no service defined 
     */
    protected URI extractAddressFromJbiDescriptor(Document jbiDoc)
        throws Exception {
        URI uri = null;

        if (jbiDoc != null) {
            Node addressNode = XMLHelper.findChild(jbiDoc, EXTENSIONS_XMLNS,
                JBI_ADDRESS_TAG, true);

            if (addressNode != null) {
                uri = new URI(addressNode.getTextContent());
            }
        }
        if (uri == null) {
            throw new Exception(
                "No Service Address was found in the jbi.xml document.");
        }
        return uri;
    }

    /**
     * Return a Document from the jbi.xml file in the given directory
     * @param serviceUnitRootPath directory where the jbi.xml can be found
     * @return a <code>Document</code>
     * @throws Exception jbi.xml file not found, or can not create a Document from it. 
     */
    protected Document findJBIDescription(String serviceUnitRootPath)
        throws Exception {
        File jbiXmlFile = new File(serviceUnitRootPath + File.separator
            + "META-INF" + File.separator + "jbi.xml");

        return WSDLHelper.createDocumentFromWSDL(jbiXmlFile);
    }

    /**
     * Construct a <code>Properties</code> object containing :
     * <br>java.naming.factory.host,
     * <br>java.naming.factory.port,
     * <br>java.naming.factory.initial
     * <br>
     * retreived from the <code>URI</code> 
     * @param jmsAddress
     * @return a Properties object well filled, non null, non empty
     * @throws MalformedURLException one of the 3 information can not be retreived from the URI
     */
    protected Properties extractJNDIProperties(URI jmsAddress)
        throws MalformedURLException {
        String host = jmsAddress.getHost();
        String port = "" + jmsAddress.getPort();
        String scheme = jmsAddress.getScheme();

        String parameters = jmsAddress.getQuery();

        String factory = extractValueForAttribute(parameters,
            URI_INITIAL_CONTEXT_FACTORY, URI_SEPARATOR);

        if (isEmpty(scheme) || isEmpty(host) || isEmpty(factory)) {
            throw new MalformedURLException(
                "scheme, host, port or initialcontextfactory classname is not correctly specified.");
        }

        String url = scheme + "://" + host;

        if (!isEmpty(port)) {
            url += ":" + port;
        }

        Properties outProperties = new Properties();
        outProperties.setProperty(InitialContext.PROVIDER_URL, url);
        outProperties.setProperty(InitialContext.INITIAL_CONTEXT_FACTORY,
            factory);

        return outProperties;
    }

    /**
     * return the name of the topic or queue : path of the URI
     * @param address
     * @return
     */
    protected String extractName(URI address) {
        String name = address.getPath();
        if (name.startsWith("/")) {
            name = name.substring(1);
        }
        return name;
    }

    /**
     * return the qcf name : URI_TOPIC_CONNEXION_FACTORY attribute name
     * @param address
     * @return tcf name, null if not found
     */
    protected String extractTopicConnexionFactory(URI address) {
        return extractValueForAttribute(address.toString(),
            URI_TOPIC_CONNEXION_FACTORY, URI_SEPARATOR);
    }

    /**
     * return the qcf name : URI_QUEUE_CONNEXION_FACTORY attribute name
     * @param address
     * @return qcf name, null if not found
     */
    protected String extractQueueConnexionFactory(URI address) {
        return extractValueForAttribute(address.toString(),
            URI_QUEUE_CONNEXION_FACTORY, URI_SEPARATOR);
    }

    /**
     * Return an URLClassLoader object, with all Jar contained in
     * the given directory path.
     * @param rootPath directory containing the Jars
     * @return a classLoader
     * @throws MalformedURLException 
     */
    protected URLClassLoader createClassLoader(String rootPath)
        throws MalformedURLException {
        File path = new File(rootPath);

        File[] files = path.listFiles(jarFileFilter);

        ArrayList<URL> urls = new ArrayList<URL>();

        if (files != null && files.length > 0) {
            for (File file : files) {
                urls.add(file.toURL());
            }
        }
        return new URLClassLoader(urls.toArray(new URL[] {}), this.getClass()
            .getClassLoader());
    }

    class JarFileFilter implements FileFilter {
        public boolean accept(File f) {
            return f.getName().toLowerCase().endsWith(".jar");
        }
    }

    class WsdlFileFilter implements FileFilter {
        public boolean accept(File f) {
            return f.getName().toLowerCase().endsWith(".wsdl");
        }
    }
}
