/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JMSCommunication.java 601 2006-06-13 12:53:50Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.jms;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.jms.JMSException;

import javax.jms.QueueConnection;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.TextMessage;
import javax.jms.TopicConnection;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;

import org.objectweb.petals.binding.jms.su.JmsConnection;
import org.objectweb.petals.binding.jms.su.JmsServiceUnitHandler;

/**
 * This class is used to send or receive JMS text Message
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 */
public class JmsConnector {

    protected Logger logger;

    protected JmsServiceUnitHandler jmsSUHandler;

    public JmsConnector(JmsServiceUnitHandler jmsSUHandler, Logger logger) {
        this.logger = logger;

        this.jmsSUHandler = jmsSUHandler;
    }

    /**
     * Send a JMS message to the JMS queue corresponding to the
     * specified exchange's endpoint.
     * @param exchange
     * @throws Exception no JMS destination found, or problem while sending the message to JMS
     */
    public void sendJMSMessage(MessageExchange exchange) throws Exception {
        logger.log(Level.FINE, "Send a message to JMS.");

        // find the connection to use for this exchange
        JmsConnection conn = jmsSUHandler.getJmsConnection(exchange
            .getEndpoint());

        if (conn == null) {
            // No such endpoint registered
            throw new Exception(
                "No JMS destination found for the ServiceEndpoint "
                    + exchange.getEndpoint().getEndpointName());
        }

        // send to a topic or a queue, depending of the connection
        if (conn.isTopicConnection()) {
            sendTopicMessage(exchange, conn);
        } else {
            sendQueueMessage(exchange, conn);
        }
    }

    /**
     * Send a message to the JMS topic corresponding to the
     * specified exchange's endpoint.
     * @param exchange
     * @param connection
     * @throws JMSException problem with JMS
     * @throws JBIException jbi->jms transformation error
     */
    public void sendTopicMessage(MessageExchange exchange,
        JmsConnection connection) throws JMSException, JBIException {
        logger.log(Level.FINE, "Send a message to a JMS Topic.");

        // create a publisher
        TopicConnection tc = connection.getTcf().createTopicConnection();

        TopicSession ts = tc.createTopicSession(true, 0);

        TopicPublisher tpub = ts.createPublisher(connection.getTopic());

        // transform the jbi msg into a jms msg
        TextMessage msg = ts.createTextMessage();

        try {
            transform(exchange.getMessage("in"), msg);
        } catch (Exception e) {
            logger.log(Level.SEVERE,
                "Can not transform the JBI message to a JMS mesasge.", e);
            throw new JBIException(e);
        }

        // publish
        tpub.publish(msg);

        //close the publisher
        ts.commit();

        tc.close();

    }

    /**
     * Send a message to the JMS queue corresponding to the
     * specified exchange's endpoint.
     * @param exchange
     * @param connection
     * @throws JMSException problem with JMS
     * @throws JBIException jbi->jms transformation error
     */
    public void sendQueueMessage(MessageExchange exchange,
        JmsConnection connection) throws JMSException, JBIException {
        logger.log(Level.FINE, "Send a message to a JMS Queue.");

        // create a sender
        QueueConnection qc = connection.getQcf().createQueueConnection();

        QueueSession qs = qc.createQueueSession(true, 0);

        QueueSender qsend = qs.createSender(connection.getQueue());

        // transform the jbi msg into a jms msg
        TextMessage msg = qs.createTextMessage();

        try {
            transform(exchange.getMessage("in"), msg);
        } catch (Exception e) {
            logger.log(Level.SEVERE,
                "Can not transform the JBI message to a JMS mesasge.", e);
            throw new JBIException(e);
        }

        // send
        qsend.send(msg);

        //close the sender
        qs.commit();

        qc.close();
    }

    /**
     * Transform the NormalizedMessage of a JBI MsgExch 
     * into a TextMessage.
     * The content , properties and attachments 
     * of the IN NormalizedMessage are set. 
     * @param jbi
     * @param jms
     * @throws JMSException writing properties to the TextMessage  
     * @throws TransformerException NormalizedMessage content access error
     * @throws IOException NormalizedMessage content access error
     */
    protected void transform(NormalizedMessage jbi, TextMessage jms)
        throws JMSException, TransformerException, IOException {
        // set normalized properties
        for (Iterator it = jbi.getPropertyNames().iterator(); it.hasNext();) {
            String pName = (String) it.next();
            jms.setObjectProperty(pName, jbi.getProperty(pName));
        }

        // set content
        StreamResult stream = new StreamResult(new ByteArrayOutputStream());
        Transformer transformer = TransformerFactory.newInstance()
            .newTransformer();
        transformer.transform(jbi.getContent(), stream);
        stream.getOutputStream().close();
        jms.setText(stream.getOutputStream().toString());

        // set attachment
        /*
         Set attachmentNames = jbi.getAttachmentNames();
         if(attachmentNames != null)
         {
         for (Iterator iter = attachmentNames.iterator(); iter.hasNext();) 
         {
         String name= (String) iter.next();
         DataHandler data = jbi.getAttachment(name);
         //TODO attachment
         }
         }*/
    }
}
