/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: IncomingJmsServiceUnitHandler.java 601 2006-06-13 12:53:50Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.jms.incoming;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;
import javax.jbi.messaging.DeliveryChannel;

import org.objectweb.petals.binding.jms.JbiConnectionFactory;
import org.objectweb.petals.binding.jms.JmsConnection;
import org.objectweb.petals.binding.jms.JmsConnectionFactory;
import org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.xmlmanipulation.ManagementMessageUtil;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.tools.jbicommon.descriptor.Services;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * This class is used to create JMS listeners on Queue or Topic
 * in order to send messages to the JBI environment
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 */
public class IncomingJmsServiceUnitHandler extends PetalsServiceUnitHandler {

    protected HashMap<String, JmsListeningSession> jmsListeningSessions;

    public static final String TYPE = "jms";

    public static final String EXTENSIONS_XMLNS = "http://petals.objectweb.org/extensions";

    public static final String TAG_TYPE = "type";

    protected Logger logger;

    protected ComponentContext context;

    protected DeliveryChannel channel;

    public IncomingJmsServiceUnitHandler(ComponentContext context,
        DeliveryChannel channel, Logger logger) {
        this.logger = logger;
        this.context = context;
        this.channel = channel;

        jmsListeningSessions = new HashMap<String, JmsListeningSession>();
    }

    /**
     * TODO add customized message
     *  (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#deploy(java.lang.String, java.lang.String, java.lang.String)
     */
    public String deploy(String serviceUnitName, String serviceUnitType,
        String serviceUnitRootPath) throws DeploymentException {

        logger.log(Level.FINE, "deploy");

        String result = null;

        if (jmsListeningSessions.containsKey(serviceUnitName)) {
            // already bound
            result = ManagementMessageUtil.getComponentTaskResult(context
                .getComponentName(), "deploy",
                ManagementMessageUtil.TASK_RESULT_FAILED);
        } else {
            try {
                // test if the SU is supported 
                if (isSupportedTypeServiceUnit(serviceUnitRootPath)) {
                    JmsConnection jmsConn = JmsConnectionFactory.instance()
                        .createConnection(serviceUnitRootPath);

                    JbiConnection jbiConn = JbiConnectionFactory.instance()
                        .createConsumesConnection(serviceUnitRootPath, logger);

                    JmsListeningSession session = new JmsListeningSession(
                        context, channel, jmsConn, jbiConn, logger);

                    jmsListeningSessions.put(serviceUnitName, session);

                    logger.log(Level.INFO, "deploy  serviceUnitName  "
                        + serviceUnitName + "  serviceUnitRootPath "
                        + serviceUnitRootPath);

                    result = ManagementMessageUtil.getComponentTaskResult(
                        context.getComponentName(), "deploy",
                        ManagementMessageUtil.TASK_RESULT_SUCCESS);
                } else {
                    // not supported service unit, return null
                }
            } catch (Exception e1) {
                logger.log(Level.SEVERE,
                    "Exception occured during deployment.", e1);

                result = ManagementMessageUtil.getComponentTaskResult(context
                    .getComponentName(), "deploy",
                    ManagementMessageUtil.TASK_RESULT_FAILED);
            }
        }
        return result;
    }

    /**
     * Init the JmsListeningSession object related to the serviceUnitName
     */
    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {

        JmsListeningSession session = jmsListeningSessions.get(serviceUnitName);

        if (session != null) {
            try {
                session.init();
            } catch (Exception e) {
                throw new DeploymentException("Can not init serviceUnit '"
                    + serviceUnitName + "'.", e);
            }
        } else {
            throw new DeploymentException("Can not init serviceUnit '"
                + serviceUnitName + "'  : not deployed");
        }

    }

    /**
     * Start listening on the Queue or Topic, related to the serviceUnitName
     * When a Jms message is be read, a JBI MessageExchange will be sent
     * on the DeliveryChannel
     */
    public void start(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        logger.log(Level.FINE, "start  serviceUnitName  " + serviceUnitName);

        JmsListeningSession session = jmsListeningSessions.get(serviceUnitName);

        if (session != null) {
            try {
                session.listen();
            } catch (Exception e) {
                throw new DeploymentException("Can not start serviceUnit '"
                    + serviceUnitName + "'.", e);
            }
        } else {
            throw new DeploymentException("Can not start serviceUnit '"
                + serviceUnitName + "'  : not deployed");
        }
    }

    public void stop(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        logger.log(Level.FINE, "stop  serviceUnitName  " + serviceUnitName);

    }

    /**
     * Close the JMS connection to the Queue or topic, related to the serviceUnitName
     */
    public void shutDown(String serviceUnitName,
        PetalsServiceUnitManager epHandler) throws DeploymentException {
        logger.log(Level.FINE, "shutdown  serviceUnitName  " + serviceUnitName);

        JmsListeningSession session = jmsListeningSessions.get(serviceUnitName);

        if (session != null) {
            try {
                session.terminate();
            } catch (Exception e) {
                throw new DeploymentException("Can not shutDown serviceUnit '"
                    + serviceUnitName + "'.", e);
            }
        } else {
            throw new DeploymentException("Can not shutDown serviceUnit '"
                + serviceUnitName + "'  : not deployed");
        }
    }

    /**
     * Unreference the JmsListeningSession reltated to the serviceUnitName
     */
    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        jmsListeningSessions.remove(serviceUnitName);

        return ManagementMessageUtil.getComponentTaskResult(context
            .getComponentName(), "undeploy",
            ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }

    /**
     * Checks if this ServiceUnitHandler can deploy this service unit
     * ( in JBI.XML : services/type = jms, 'services' element not empty )
     * @return true if the service unit type is supported or else false
     */
    protected boolean isSupportedTypeServiceUnit(String serviceUnitRootPath) {

        boolean result = false;

        File jbiXmlFile = new File(serviceUnitRootPath + File.separator
            + "META-INF" + File.separator + "jbi.xml");

        try {

            // 1. check that 'services/consumes' element is not empty
            JBIDescriptorBuilder descriptorBuilder = new JBIDescriptorBuilder(
                jbiXmlFile.toURI(), logger);
            JBIDescriptor descriptor = descriptorBuilder.build();
            Services services = descriptor.getServices();

            List consumesList = services.getConsumes();
            boolean providesOk = (consumesList != null && consumesList.size() > 0);

            // 2. check that 'services/type' element is set to 'jms'
            Document configDoc = WSDLHelper.createDocumentFromWSDL(jbiXmlFile);

            Node servicesNode = XMLHelper
                .findChild(configDoc, "services", true);

            boolean typeOk = false;

            if (servicesNode != null) {
                Node typeNode = XMLHelper.findChild(servicesNode,
                    EXTENSIONS_XMLNS, TAG_TYPE, false);

                if (typeNode != null
                    && typeNode.getNodeType() == Node.ELEMENT_NODE) {
                    typeOk = (TYPE.equalsIgnoreCase(typeNode.getTextContent()));
                }
            }
            result = providesOk && typeOk;

        } catch (SAXException e) {
            logger.log(Level.SEVERE, "Can not analyse jbi.xml file.", e);
        }

        return result;
    }
}
