/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIExchangeProcessor.java 12:18:02 alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.jms.outgoing;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.RobustInOnly;
import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;

import org.objectweb.petals.component.common.MEPConstants;
import org.objectweb.petals.component.common.listener.IMessageExchangeProcessor;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.w3c.dom.Document;

/**
 * This class is the JBI message processor for the JmsBC component.
 * It receives messages from JBI and send them to a JMS destination.
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 */
public class JBIExchangeProcessor implements IMessageExchangeProcessor {

    protected PetalsServiceUnitManager suManager;

    protected ComponentContext ctx;

    protected DeliveryChannel channel;

    protected Logger l;

    protected JmsConnector jmsConnector;

    public JBIExchangeProcessor(ComponentContext c, DeliveryChannel channel,
        PetalsServiceUnitManager suManager, JmsConnector connector, Logger log) {
        ctx = c;

        l = log;

        this.suManager = suManager;

        this.channel = channel;

        this.jmsConnector = connector;
    }

    /**
     * Process ACTIVE messages as a PROVIDER.
     * Only InOnly and RobustInOnly messages are processed.
     * Otherwise, set the Exchange to ERROR status and send it, and throw an Exception
     * 
     * If the specified exchange endpoint is not registered,
     *   set the Exchange to ERROR status and send it, and throw an Exception     * 
     * Else, 
     *   send a <em>JMS message</em> to the corresponding <em>JMS Queue<em>,
     *   
     *   If the JMS msg has not been correctly sent
     *     set the Exchange to ERROR status and send it, and throw an Exception     *
     *      
     * @param exchange
     * @throws Exception exhange not correct, destination not found, problem while sending the JMS message,
     */
    public boolean process(MessageExchange exchange) throws Exception {
        l.log(Level.FINE, "process");

        if (ExchangeStatus.ACTIVE.equals(exchange.getStatus())) {
            if (MessageExchange.Role.PROVIDER.equals(exchange.getRole())) {
                if (MEPConstants.IN_ONLY_PATTERN.equals(exchange.getPattern())
                    || MEPConstants.ROBUST_IN_ONLY_PATTERN.equals(exchange
                        .getPattern())) {
                    try {
                        // 1. send a JMSMessage to the corresponding queue
                        jmsConnector.sendJMSMessage(exchange);

                        // 2a. the JMS message has been sent,
                        //     send a Done status
                        sendDone(exchange);
                    } catch (Exception e) {
                        // 2b. error, set an error status
                        sendError(exchange, e, true);
                    }
                } else {
                    Exception e = new Exception(
                        "MessageExchangePattern not recognized :"
                            + exchange.getPattern());
                    sendError(exchange, e, true);
                }
            }
        }
        return true;
    }

    /**
     * Send a response message with a status set to "DONE".
     * 
     * @param msgExchange
     *            current MessageExchange used to send and receive messages
     * @throws MessagingException
     *             If an error occured during calls on the JBI container
     */
    protected void sendDone(MessageExchange msgExchange)
        throws MessagingException {
        msgExchange.setStatus(ExchangeStatus.DONE);
        channel.send(msgExchange);
    }

    /**
     * Send a response message with status set to "ERROR" to the service
     * consumer. An exception is created with the given reason as message. This
     * exception is set in the MessageExchange object. The modified
     * MessageExchange is then returned to the service consumer.
     * 
     * @param msgExchange
     *            current MessageExchange used to send and receive messages
     * @param ex
     *            The exception
     * @param severe
     *            Log the error at warning or severe level
     * @throws MessagingException
     *             If an error occured during calls on the JBI container
     */
    protected void sendError(MessageExchange msgExchange, Exception ex,
        boolean severe) throws MessagingException {
        msgExchange.setError(ex);
        msgExchange.setStatus(ExchangeStatus.ERROR);
        channel.send(msgExchange);
        if (!severe) {
            l.log(Level.WARNING, ex.getMessage(), ex);
        } else {
            l.log(Level.SEVERE, ex.getMessage(), ex);
        }
    }

    /**
     * Send a response message with status set to "ERROR" to the service
     * consumer. A fault is created with the given reason as content. This fault
     * is set in the MessageExchange object. The modified MessageExchange is
     * then returned to the service consumer.
     * 
     * @param msgExchange
     *            current MessageExchange used to send and receive messages
     * @param reason
     *            The reason why the error was raised
     * @param e
     *            The exception which created the fault
     * @throws MessagingException
     *             If an error occured during calls on the JBI container
     */
    protected void sendFault(MessageExchange msgExchange, String reason,
        Exception e) throws MessagingException {
        if ((msgExchange instanceof InOnly)
            && !(msgExchange instanceof RobustInOnly)) {
            sendDone(msgExchange);
        } else {
            Fault fault = msgExchange.createFault();
            String faultString = createSoapFault(e, reason);
            fault.setContent(createSource(faultString));
            msgExchange.setFault(fault);
            channel.send(msgExchange);
        }
    }

    protected String createSoapFault(Exception e, String reason) {
        return SourceHelper.createSoapFault(e, reason);
    }

    /**
     * 
     * @see org.objectweb.petals.component.common.xmlmanipulation.SourceHelper#createSource(java.lang.Source)
     */
    protected Source createSource(String input) {
        return SourceHelper.createSource(input);
    }

    /**
     * 
     * @see org.objectweb.petals.component.common.xmlmanipulation.SourceHelper#createString(javax.xml.transform.Source)
     */
    protected String createString(Source source) throws Exception {
        return SourceHelper.createString(source);
    }

    /**
     * 
     * @throws WSDLException
     * @throws TransformerException 
     * @see org.objectweb.petals.component.common.xmlmanipulation.XMLHelper#getPortNameFromWSDLDocument(Document)(org.w3c.dom.Document )
     */
    protected String getPortNameFromWSDLDocument(Document serviceDesc,
        QName serviceName) throws WSDLException, TransformerException {
        return WSDLHelper.getPortNameFromWSDLDocument(serviceDesc, serviceName);
    }
}
