/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SUDeploymentManager.java 154 31 juil. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.net.URI;

import javax.jbi.JBIException;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;

import org.objectweb.petals.jbi.management.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.ServiceUnitLifeCycle;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.tools.jbicommon.descriptor.Services;
import org.objectweb.petals.util.LoggingUtil;

/**
 * SUDeploymentManager is used to handle the whole process of a specific service
 * unit deployment.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SUDeploymentManager {

    /**
     * The platform deployment service
     */
    protected DeploymentService deploymentService;

    /**
     * The SU installation root {@link URI}
     */
    protected URI installationRoot;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected LifeCycleManagerService managerService;

    /**
     * Platform util : package handler
     */
    protected PackageHandler packageHandler;

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    /**
     * The parent service assembly lifecycle
     */
    protected ServiceAssemblyLifeCycle saLifeCycle;

    /**
     * The parent service assembly name
     */
    protected String saName;

    /**
     * Services activated by this service unit
     */
    protected Services services;

    /**
     * The service assembly production element
     */
    protected ServiceUnit serviceUnit;

    public SUDeploymentManager() {
        super();
    }

    /**
     * 
     * @param serviceUnit
     *            the {@link ServiceUnit} production element
     * @param installationRoot
     *            the SU installation root {@link URI}
     * @param saLifeCycle
     *            the life cycle of the parent SA
     * @param repositorySrv
     *            the platform {@link RepositoryService}
     * @param managerService
     *            the platform {@link LifeCycleManagerService}
     * @param log
     *            the platform {@link LoggingUtil}
     * @param packageHandler
     *            the platform {@link PackageHandler} util
     * @param deploymentService
     *            the platform {@link DeploymentService}
     */
    public SUDeploymentManager(ServiceUnit serviceUnit, URI installationRoot,
            ServiceAssemblyLifeCycle saLifeCycle,
            RepositoryService repositorySrv,
            LifeCycleManagerService managerService, LoggingUtil log,
            PackageHandler packageHandler, DeploymentService deploymentService) {
        super();
        this.serviceUnit = serviceUnit;
        this.installationRoot = installationRoot;
        this.saLifeCycle = saLifeCycle;
        this.repositorySrv = repositorySrv;
        this.managerService = managerService;
        this.log = log;
        this.packageHandler = packageHandler;
        this.deploymentService = deploymentService;
    }

    /**
     * Process the deployment of the related service unit.
     * 
     * @return a {@link String} xml result of the service unit deployment (see
     *         section 6.9 of the JSR 208)
     * @throws ManagementException
     *             when service unit lifecycle registration failed. It's is
     *             considered as a severe system error and will be treated as a
     *             complete service assembly deployment failure.
     */
    public String deploySU(boolean recovery) throws ManagementException {

        String xmlResult = null;

        /*
         * 1. Deploy service unit to the targeted component
         */
        xmlResult = deploySUtoComponent(installationRoot);

        /*
         * 2. Create service unit lifecycle
         */
        ServiceUnitLifeCycle suLifeCycle = createNewServiceUnitLifeCycle(installationRoot
                .getPath());

        /*
         * 3. Register this service unit lifecycle into the sa lifecycle
         */
        registerSULifeCycle(suLifeCycle);

        return xmlResult;
    }

    /**
     * Process the undeployment of the related service unit.
     * 
     * @return a {@link String} xml result of the service unit undeployment (see
     *         section 6.9 of the JSR 208)
     * @throws Exception
     *             if an error occurs during the undeployment process : can't
     *             retrieve the {@link ComponentLifeCycle}, can't retrieve the
     *             component {@link ServiceUnitManager} (SUM) or an exception is
     *             thrown during the call of the undeploy method of the SUM
     */
    public String undeploySU() throws Exception {
        String msg = "";
        String xmlReturn = null;

        /*
         * Undeploy it
         */
        String suName = getServiceUnitName();
        ServiceUnitLifeCycle serviceUnitLifeCycle = saLifeCycle
                .getServiceUnitsLifeCycles().get(suName);
        String targetedComponent = serviceUnit.getTargetComponentName();
        // Retrieve the component life cycle
        ComponentLifeCycle clc = managerService
                .getComponentByName(targetedComponent);
        if (clc == null) {
            msg = "The component ("
                    + targetedComponent
                    + ") is not deployed. The service unit cannot be undeployed.";
            log.error(msg);
            throw new Exception(msg);
        }
        // Retrieve the service unit manager of the component
        ServiceUnitManager sum = clc.getComponent().getServiceUnitManager();
        if (sum == null) {
            msg = "The service unit manager is null for the component ("
                    + targetedComponent
                    + "). You may need to initialise the component. The service units cannot be undeployed.";
            log.error(msg);
            throw new Exception(msg);
        }

        try {
            xmlReturn = sum.undeploy(suName, serviceUnitLifeCycle
                    .getServiceUnitRootPath());
        } catch (DeploymentException e) {
            msg = "Service unit manager of the component (" + targetedComponent
                    + ") failed to undeploy su (" + suName + ")";
            log.error(msg);
            throw new Exception(msg, e);
        }

        /*
         * Remove the service unit from the service assembly
         */
        saLifeCycle.getServiceUnitsLifeCycles().remove(suName);

        return xmlReturn;
    }

    /**
     * Creates a new {@link ServiceUnitLifeCycle}
     * 
     * @param rootPath
     *            the SU installation root
     * @return a new {@link ServiceUnitLifeCycle} instance
     */
    protected ServiceUnitLifeCycle createNewServiceUnitLifeCycle(String rootPath) {
        return new ServiceUnitLifeCycle(serviceUnit, managerService, rootPath,
                log);
    }

    /**
     * Deploy the given service unit to the targeted component
     * 
     * @param installationRoot
     *            the su installation root {@link URI}
     * @return a {@link String} xml result of the service unit deployment on the
     *         targeted component (see section 6.9 of the JSR 208)
     * @throws ManagementException
     *             if component and related object are missing or if the call of
     *             the service unit manager method "deploy" fails
     */
    protected String deploySUtoComponent(URI installationRoot)
        throws ManagementException {

        /*
         * Retrieve the target component life cycle
         */
        String targetedComponent = serviceUnit.getTargetComponentName();
        ComponentLifeCycle clc = managerService
                .getComponentByName(targetedComponent);
        if (clc == null) {
            String msg = "The targeted component lifecycle element for service unit deployment must be non null.";
            log.error(msg);
            throw new ManagementException(msg);
        }

        /*
         * Retrieve the service unit manager of the component that allow to
         * process deployment
         */
        ServiceUnitManager sum = clc.getComponent().getServiceUnitManager();
        if (sum == null) {
            String msg = "Component "
                    + targetedComponent
                    + " doesn't support deployment : service unit manager is null !";
            log.error(msg);
            throw new ManagementException(msg);
        }

        String xmlReturn = null;
        try {
            xmlReturn = sum.deploy(getServiceUnitName(), installationRoot
                    .getPath());
        } catch (DeploymentException e) {
            String msg = "Deployement of the service unit "
                    + getServiceUnitName()
                    + " has failed (caused by the targeted component \""
                    + getTargetedComponent() + "\" su manager exception).";
            log.error(msg);
            throw new ManagementException(msg, e);
        }
        return xmlReturn;
    }

    /**
     * Retrieve the name of the parent service assembly
     * 
     * @return the parent service assembly name
     */
    protected String getParentServiceAssemblyName() {
        return saLifeCycle.getServiceAssembly().getIdentification().getName();
    }

    /**
     * Retrieve the service unit name found in the SU production element
     * 
     * @return the service unit name
     */
    protected String getServiceUnitName() {
        return serviceUnit.getIdentification().getName();
    }

    /**
     * Retrieve the service unit targeted component
     * 
     * @return the service unit targeted component name
     */
    protected String getTargetedComponent() {
        return serviceUnit.getTargetComponentName();
    }

    /**
     * Register the {@link ServiceUnitLifeCycle} to the
     * {@link ServiceAssemblyLifeCycle}.
     * 
     * @param suLifeCycle
     *            the {@link ServiceUnitLifeCycle} to register
     * @throws ManagementException
     *             if it fails to register {@link ServiceUnitLifeCycle} to
     *             {@link ServiceAssemblyLifeCycle}
     */
    protected void registerSULifeCycle(ServiceUnitLifeCycle suLifeCycle)
        throws ManagementException {
        try {
            saLifeCycle.registerSUIntoSA(getServiceUnitName(), suLifeCycle);
        } catch (JBIException e) {
            String msg = "Error while registering the service unit "
                    + getServiceUnitName()
                    + " life cycle into the service assembly life cycle"
                    + saName + ".";
            log.error(msg);
            throw new ManagementException(msg, e);
        }
    }

}
