/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: JoramQueue.java,v 0.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.transport;

import java.net.ConnectException;
import java.net.UnknownHostException;

import javax.jbi.JBIException;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.joram.client.jms.tcp.QueueTcpConnectionFactory;
import org.objectweb.joram.client.jms.tcp.TcpConnectionFactory;
import org.objectweb.petals.util.JNDIUtil;

/**
 * @author Adrien LOUIS - EBM WebSourcing
 * 
 */
public class JoramQueue {

    protected static final String CONNECTION_FACTORY = "JoramConnectionFactory";

    protected static final String QUEUE_CONNECTION_FACTORY = "JoramQueueConnectionFactory";

    protected static final String QUEUE_NAME = "JoramQueue-";

    protected String host;

    protected InitialContext ictx;

    protected int id;

    protected String pwd;

    protected QueueConnectionFactory qcf;

    protected QueueSession qs;

    protected Queue queue;

    protected int tcp;

    protected String user;

    public JoramQueue(int id, int tcp, String user, String pwd, String host,
        InitialContext ictx) {
        this.id = id;
        this.tcp = tcp;
        this.user = user;
        this.pwd = pwd;
        this.host = host;
        this.ictx = ictx;
    }

    public static String containerNameToQueueName(String containerName) {
        return QUEUE_NAME + containerName;
    }

    public QueueConnectionFactory getQueueConnectionFactory() {
        return qcf;
    }

    /**
     * init global connectionfactory, queueconnectionfactory, and the queue for
     * this server, by connecting to the joram AdminModule.
     * 
     * @throws JBIException
     */
    public void init() throws JBIException {
        try {
            connectAdminModule();

            setupConnectionFactory();

            qcf = setupQueueConnectionFactory();

            queue = setupQueue();

            disconnectAdminModule();
        } catch (Exception e) {
            throw new JBIException(e);
        }
    }

    /**
     * Start listening on the queue of this server. the specified
     * MessageListener is called when a message incomes. prereq : init()
     * 
     * @param aML
     * @throws JMSException
     */
    public void startListening(MessageListener aML) throws JMSException {
        QueueConnection qc = qcf.createQueueConnection(user, pwd);
        qs = qc.createQueueSession(true, 0);
        QueueReceiver qrec = qs.createReceiver(queue);
        qrec.setMessageListener(aML);
        qc.start();
    }

    /**
     * Stop listening. The queuesession is closed
     * 
     * @throws JMSException
     */
    public void stopListening() throws JMSException {
        qs.commit();

        qs.close();
    }

    public void unbindQueue() throws JBIException {
        try {
            ictx.unbind(QUEUE_NAME + id);
        } catch (NamingException ex) {
            throw new JBIException(ex);
        }
    }

    /**
     * @see AdminModule#connect(String, int, String, String, int, String)
     */
    protected void connectAdminModule() throws ConnectException,
        UnknownHostException, AdminException {
        AdminModule.connect(host, tcp, user, pwd, 3);
    }

    /**
     * @see TcpConnectionFactory#create()
     */
    protected ConnectionFactory createConnectionFactory()
        throws ConnectException {
        return TcpConnectionFactory.create();
    }

    /**
     * @see Queue#create(String)
     */
    protected Queue createQueue(String queueName) throws Exception {
        return Queue.create(queueName);
    }

    /**
     * @see QueueTcpConnectionFactory#create()
     */
    protected QueueConnectionFactory createQueueConnectionFactory()
        throws ConnectException {
        return QueueTcpConnectionFactory.create();
    }

    /**
     * @see AdminModule#disconnect()
     */
    protected void disconnectAdminModule() {
        AdminModule.disconnect();
    }

    /**
     * setup the global ConnectionFactory in the distributed jndi (create or
     * retrieve it) Connection to the Joram AdminModule must be done
     * 
     * @return the global QueueConnection
     * @throws NamingException
     * @throws ConnectException
     */
    protected ConnectionFactory setupConnectionFactory()
        throws NamingException, ConnectException {
        ConnectionFactory cFact = null;

        boolean alreadyBound = JNDIUtil.isBound(this.ictx, "/",
            CONNECTION_FACTORY);

        if (alreadyBound) {
            cFact = (ConnectionFactory) ictx.lookup(CONNECTION_FACTORY);
        } else {
            cFact = createConnectionFactory();

            ictx.bind(CONNECTION_FACTORY, cFact);
        }
        return cFact;
    }

    /**
     * Setup the queue for this server in the global JNDI (create or retrieve
     * it). Connection to the Joram AdminModule must be done
     * 
     * @return the queue
     */
    protected Queue setupQueue() throws Exception {
        Queue q = null;

        String queueName = QUEUE_NAME + id;

        boolean alreadyBound = JNDIUtil.isBound(ictx, "/", queueName);

        if (alreadyBound) {
            q = (Queue) ictx.lookup(queueName);
        } else {
            q = createQueue(queueName);

            q.setFreeReading();

            q.setFreeWriting();

            ictx.bind(queueName, q);
        }
        return q;
    }

    /**
     * setup the global QueueConnectionFactory in the distributed jndi (create
     * or retrieve it). Connection to the Joram AdminModule must be done
     * 
     * @return the global QueueConnectionFactory
     * @throws NamingException
     * @throws ConnectException
     */
    protected QueueConnectionFactory setupQueueConnectionFactory()
        throws NamingException, ConnectException {
        QueueConnectionFactory qcFact = null;

        boolean alreadyBound = JNDIUtil.isBound(this.ictx, "/",
            QUEUE_CONNECTION_FACTORY);

        if (alreadyBound) {
            qcFact = (QueueConnectionFactory) ictx
                .lookup(QUEUE_CONNECTION_FACTORY);
        } else {
            qcFact = createQueueConnectionFactory();

            ictx.bind(QUEUE_CONNECTION_FACTORY, qcFact);
        }
        return qcFact;
    }

}
