/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIDescriptor.java 477 2006-05-29 15:18:07Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbicommon.descriptor;


/**
 * JBI Descriptor.
 * 
 * @version $Rev: 477 $ $Date: 2006-05-29 15:18:07Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class JBIDescriptor {

    // ----------------------------------------------------------------------
    // constants and inner classes declarations
    // ----------------------------------------------------------------------

    /**
     * ClassLoader parent first delegation option (<code>"parent-first"</code>).
     */
    private static final String CL_DELEGATION_PARENT_FIRST = "parent-first";

    /**
     * ClassLoader self first delegation option (<code>"self-first"</code>).
     */
    private static final String CL_DELEGATION_SELF_FIRST = "self-first";

    /**
     * JBI Component production.
     */
    protected ComponentDescription component;

    /**
     * Service Assembly production.
     */
    protected ServiceAssembly serviceAssembly;

    /**
     * Services production.
     */
    protected Services services;

    /**
     * Shared Library production.
     */
    protected SharedLibrary sharedLibrary;

    /**
     * JBI Descriptor version.
     */
    private double version;

    /**
     * Default constructor.
     */
    public JBIDescriptor() {
        super();
    }

    // ----------------------------------------------------------------------
    // Getters and Setters
    // ----------------------------------------------------------------------

    /**
     * Return true if the delegation model is "parent-first" or null (default)
     * Return false otherwise
     */
    public static boolean isParentFirst(String delegation) {
        return (delegation == null)
                || (CL_DELEGATION_PARENT_FIRST.equals(delegation));
    }

    /**
     * Return true if the delegation model is "self-first" Return false
     * otherwise
     */
    public static boolean isSelfFirst(String delegation) {
        return CL_DELEGATION_SELF_FIRST.equals(delegation);

    }

    /**
     * Returns the component production loaded from JBI Descriptor.
     * 
     * @return Returns the component.
     */
    public ComponentDescription getComponent() {
        return component;
    }

    /**
     * Returns the service assembly production loaded from JBI Descriptor.
     * 
     * @return Returns the serviceAssembly.
     */
    public ServiceAssembly getServiceAssembly() {
        return serviceAssembly;
    }

    /**
     * Returns the services production loaded from JBI Descriptor.
     * 
     * @return Returns the services.
     */
    public Services getServices() {
        return services;
    }

    /**
     * Returns the shared library production from JBI Descriptor.
     * 
     * @return Returns the sharedLibrary.
     */
    public SharedLibrary getSharedLibrary() {
        return sharedLibrary;
    }

    /**
     * Attribute version is a decimal value indicating the version of descriptor
     * in use.
     * 
     * @return <code>double</code> the version.
     */
    public double getVersion() {
        return version;
    }

    /**
     * Set the component JBI Descriptor production loaded from a XML.
     * 
     * @param component
     *            the component production.
     */
    protected void setComponent(ComponentDescription component) {
        this.component = component;
    }

    /**
     * Set the service assembly JBI Descriptor production loaded from a XML.
     * 
     * @param serviceAssembly
     *            the service assembly production.
     */
    protected void setServiceAssembly(ServiceAssembly serviceAssembly) {
        this.serviceAssembly = serviceAssembly;
    }

    /**
     * Set the services JBI Descriptor production loaded from a XML.
     * 
     * @param services
     *            the services production.
     */
    protected void setServices(Services services) {
        this.services = services;
    }

    // ----------------------------------------------------------------------
    // static Descriptor utility methods
    // ----------------------------------------------------------------------

    /**
     * Set the shared library JBI Descriptor production loaded from a XML.
     * 
     * @param sharedLibrary
     *            the shared library production.
     */
    protected void setSharedLibrary(SharedLibrary sharedLibrary) {
        this.sharedLibrary = sharedLibrary;
    }

    /**
     * Attribute version is a decimal value indicating the version of descriptor
     * in use.
     * 
     * @param version
     *            The version <code>double</code> value to set.
     */
    protected void setVersion(double version) {
        this.version = version;
    }
}
